@@ -6,7 +6,10 @@
 
 #include "core/fpdfapi/page/cpdf_pathobject.h"
 
-CPDF_PathObject::CPDF_PathObject() {}
+CPDF_PathObject::CPDF_PathObject(int32_t content_stream)
+    : CPDF_PageObject(content_stream), m_FillType(0), m_bStroke(false) {}
+
+CPDF_PathObject::CPDF_PathObject() : CPDF_PathObject(kNoContentStream) {}
 
 CPDF_PathObject::~CPDF_PathObject() {}
 
@@ -17,6 +20,7 @@ CPDF_PageObject::Type CPDF_PathObject::GetType() const {
 void CPDF_PathObject::Transform(const CFX_Matrix& matrix) {
   m_Matrix.Concat(matrix);
   CalcBoundingBox();
+  SetDirty(true);
 }
 
 bool CPDF_PathObject::IsPath() const {
@@ -32,7 +36,7 @@ const CPDF_PathObject* CPDF_PathObject::AsPath() const {
 }
 
 void CPDF_PathObject::CalcBoundingBox() {
-  if (!m_Path)
+  if (!m_Path.HasRef())
     return;
   CFX_FloatRect rect;
   float width = m_GraphState.GetLineWidth();
@@ -41,7 +45,7 @@ void CPDF_PathObject::CalcBoundingBox() {
   } else {
     rect = m_Path.GetBoundingBox();
   }
-  m_Matrix.TransformRect(rect);
+  rect = m_Matrix.TransformRect(rect);
 
   if (width == 0 && m_bStroke) {
     rect.left += -0.5f;
