@@ -17,8 +17,8 @@
 #include "core/fpdfapi/parser/cpdf_crypto_handler.h"
 #include "core/fpdfapi/parser/cpdf_dictionary.h"
 #include "core/fpdfapi/parser/cpdf_object.h"
-#include "core/fpdfapi/parser/cpdf_parser.h"
 #include "core/fpdfapi/parser/cpdf_string.h"
+#include "core/fxcrt/fx_extension.h"
 
 namespace {
 
@@ -27,29 +27,28 @@ const uint8_t defpasscode[32] = {
     0x56, 0xff, 0xfa, 0x01, 0x08, 0x2e, 0x2e, 0x00, 0xb6, 0xd0, 0x68,
     0x3e, 0x80, 0x2f, 0x0c, 0xa9, 0xfe, 0x64, 0x53, 0x69, 0x7a};
 
-void CalcEncryptKey(CPDF_Dictionary* pEncrypt,
-                    const uint8_t* password,
-                    uint32_t pass_size,
+void CalcEncryptKey(const CPDF_Dictionary* pEncrypt,
+                    const ByteString& password,
                     uint8_t* key,
                     int keylen,
                     bool bIgnoreMeta,
-                    CPDF_Array* pIdArray) {
+                    const ByteString& fileId) {
   int revision = pEncrypt->GetIntegerFor("R");
   uint8_t passcode[32];
   for (uint32_t i = 0; i < 32; i++) {
-    passcode[i] = i < pass_size ? password[i] : defpasscode[i - pass_size];
+    passcode[i] = i < password.GetLength()
+                      ? password[i]
+                      : defpasscode[i - password.GetLength()];
   }
   CRYPT_md5_context md5;
   CRYPT_MD5Start(&md5);
   CRYPT_MD5Update(&md5, passcode, 32);
-  CFX_ByteString okey = pEncrypt->GetStringFor("O");
+  ByteString okey = pEncrypt->GetStringFor("O");
   CRYPT_MD5Update(&md5, (uint8_t*)okey.c_str(), okey.GetLength());
   uint32_t perm = pEncrypt->GetIntegerFor("P");
   CRYPT_MD5Update(&md5, (uint8_t*)&perm, 4);
-  if (pIdArray) {
-    CFX_ByteString id = pIdArray->GetStringAt(0);
-    CRYPT_MD5Update(&md5, (uint8_t*)id.c_str(), id.GetLength());
-  }
+  if (!fileId.IsEmpty())
+    CRYPT_MD5Update(&md5, (uint8_t*)fileId.c_str(), fileId.GetLength());
   if (!bIgnoreMeta && revision >= 3 &&
       !pEncrypt->GetIntegerFor("EncryptMetadata", 1)) {
     uint32_t tag = 0xFFFFFFFF;
@@ -58,25 +57,37 @@ void CalcEncryptKey(CPDF_Dictionary* pEncrypt,
   uint8_t digest[16];
   CRYPT_MD5Finish(&md5, digest);
   uint32_t copy_len = keylen;
-  if (copy_len > sizeof(digest)) {
+  if (copy_len > sizeof(digest))
     copy_len = sizeof(digest);
-  }
   if (revision >= 3) {
-    for (int i = 0; i < 50; i++) {
+    for (int i = 0; i < 50; i++)
       CRYPT_MD5Generate(digest, copy_len, digest);
-    }
   }
   memset(key, 0, keylen);
   memcpy(key, digest, copy_len);
 }
 
+bool IsValidKeyLengthForCipher(int cipher, int keylen) {
+  switch (cipher) {
+    case FXCIPHER_AES:
+      return keylen == 16 || keylen == 24 || keylen == 32;
+    case FXCIPHER_AES2:
+      return keylen == 32;
+    case FXCIPHER_RC4:
+      return keylen >= 5 && keylen <= 16;
+    case FXCIPHER_NONE:
+      return true;
+    default:
+      NOTREACHED();
+  }
+  return false;
+}
+
 }  // namespace
 
 CPDF_SecurityHandler::CPDF_SecurityHandler()
     : m_Version(0),
       m_Revision(0),
-      m_pParser(nullptr),
-      m_pEncryptDict(nullptr),
       m_Permissions(0),
       m_Cipher(FXCIPHER_NONE),
       m_KeyLen(0),
@@ -84,57 +95,59 @@ CPDF_SecurityHandler::CPDF_SecurityHandler()
 
 CPDF_SecurityHandler::~CPDF_SecurityHandler() {}
 
-CPDF_CryptoHandler* CPDF_SecurityHandler::CreateCryptoHandler() {
-  return new CPDF_CryptoHandler;
-}
-
-bool CPDF_SecurityHandler::OnInit(CPDF_Parser* pParser,
-                                  CPDF_Dictionary* pEncryptDict) {
-  m_pParser = pParser;
-  if (!LoadDict(pEncryptDict)) {
+bool CPDF_SecurityHandler::OnInit(const CPDF_Dictionary* pEncryptDict,
+                                  const CPDF_Array* pIdArray,
+                                  const ByteString& password) {
+  m_FileId = pIdArray ? pIdArray->GetStringAt(0) : "";
+  if (!LoadDict(pEncryptDict))
     return false;
-  }
-  if (m_Cipher == FXCIPHER_NONE) {
+  if (m_Cipher == FXCIPHER_NONE)
     return true;
-  }
-  return CheckSecurity(m_KeyLen);
+  if (!CheckSecurity(password))
+    return false;
+
+  InitCryptoHandler();
+  return true;
 }
 
-bool CPDF_SecurityHandler::CheckSecurity(int32_t key_len) {
-  CFX_ByteString password = m_pParser->GetPassword();
+bool CPDF_SecurityHandler::CheckSecurity(const ByteString& password) {
   if (!password.IsEmpty() &&
-      CheckPassword(password.raw_str(), password.GetLength(), true,
-                    m_EncryptKey, key_len)) {
+      CheckPassword(password, true, m_EncryptKey, m_KeyLen)) {
     m_bOwnerUnlocked = true;
     return true;
   }
-  return CheckPassword(password.raw_str(), password.GetLength(), false,
-                       m_EncryptKey, key_len);
+  return CheckPassword(password, false, m_EncryptKey, m_KeyLen);
 }
 
-uint32_t CPDF_SecurityHandler::GetPermissions() {
-  return m_bOwnerUnlocked ? 0xFFFFFFFF : m_Permissions;
+uint32_t CPDF_SecurityHandler::GetPermissions() const {
+  uint32_t dwPermission = m_bOwnerUnlocked ? 0xFFFFFFFF : m_Permissions;
+  if (m_pEncryptDict && m_pEncryptDict->GetStringFor("Filter") == "Standard") {
+    // See PDF Reference 1.7, page 123, table 3.20.
+    dwPermission &= 0xFFFFFFFC;
+    dwPermission |= 0xFFFFF0C0;
+  }
+  return dwPermission;
 }
 
-static bool LoadCryptInfo(CPDF_Dictionary* pEncryptDict,
-                          const CFX_ByteString& name,
+static bool LoadCryptInfo(const CPDF_Dictionary* pEncryptDict,
+                          const ByteString& name,
                           int& cipher,
                           int& keylen) {
   int Version = pEncryptDict->GetIntegerFor("V");
   cipher = FXCIPHER_RC4;
   keylen = 0;
   if (Version >= 4) {
-    CPDF_Dictionary* pCryptFilters = pEncryptDict->GetDictFor("CF");
-    if (!pCryptFilters) {
+    const CPDF_Dictionary* pCryptFilters = pEncryptDict->GetDictFor("CF");
+    if (!pCryptFilters)
       return false;
-    }
+
     if (name == "Identity") {
       cipher = FXCIPHER_NONE;
     } else {
-      CPDF_Dictionary* pDefFilter = pCryptFilters->GetDictFor(name);
-      if (!pDefFilter) {
+      const CPDF_Dictionary* pDefFilter = pCryptFilters->GetDictFor(name);
+      if (!pDefFilter)
         return false;
-      }
+
       int nKeyBits = 0;
       if (Version == 4) {
         nKeyBits = pDefFilter->GetIntegerFor("Length", 0);
@@ -144,11 +157,14 @@ static bool LoadCryptInfo(CPDF_Dictionary* pEncryptDict,
       } else {
         nKeyBits = pEncryptDict->GetIntegerFor("Length", 256);
       }
+      if (nKeyBits < 0)
+        return false;
+
       if (nKeyBits < 40) {
         nKeyBits *= 8;
       }
       keylen = nKeyBits / 8;
-      CFX_ByteString cipher_name = pDefFilter->GetStringFor("CFM");
+      ByteString cipher_name = pDefFilter->GetStringFor("CFM");
       if (cipher_name == "AESV2" || cipher_name == "AESV3") {
         cipher = FXCIPHER_AES;
       }
@@ -159,27 +175,26 @@ static bool LoadCryptInfo(CPDF_Dictionary* pEncryptDict,
   if (keylen > 32 || keylen < 0) {
     return false;
   }
-  return true;
+  return IsValidKeyLengthForCipher(cipher, keylen);
 }
 
-bool CPDF_SecurityHandler::LoadDict(CPDF_Dictionary* pEncryptDict) {
+bool CPDF_SecurityHandler::LoadDict(const CPDF_Dictionary* pEncryptDict) {
   m_pEncryptDict = pEncryptDict;
   m_Version = pEncryptDict->GetIntegerFor("V");
   m_Revision = pEncryptDict->GetIntegerFor("R");
   m_Permissions = pEncryptDict->GetIntegerFor("P", -1);
   if (m_Version < 4)
-    return LoadCryptInfo(pEncryptDict, CFX_ByteString(), m_Cipher, m_KeyLen);
+    return LoadCryptInfo(pEncryptDict, ByteString(), m_Cipher, m_KeyLen);
 
-  CFX_ByteString stmf_name = pEncryptDict->GetStringFor("StmF");
-  CFX_ByteString strf_name = pEncryptDict->GetStringFor("StrF");
+  ByteString stmf_name = pEncryptDict->GetStringFor("StmF");
+  ByteString strf_name = pEncryptDict->GetStringFor("StrF");
   if (stmf_name != strf_name)
     return false;
 
   return LoadCryptInfo(pEncryptDict, strf_name, m_Cipher, m_KeyLen);
 }
 
-bool CPDF_SecurityHandler::LoadDict(CPDF_Dictionary* pEncryptDict,
-                                    uint32_t type,
+bool CPDF_SecurityHandler::LoadDict(const CPDF_Dictionary* pEncryptDict,
                                     int& cipher,
                                     int& key_len) {
   m_pEncryptDict = pEncryptDict;
@@ -187,8 +202,8 @@ bool CPDF_SecurityHandler::LoadDict(CPDF_Dictionary* pEncryptDict,
   m_Revision = pEncryptDict->GetIntegerFor("R");
   m_Permissions = pEncryptDict->GetIntegerFor("P", -1);
 
-  CFX_ByteString strf_name;
-  CFX_ByteString stmf_name;
+  ByteString strf_name;
+  ByteString stmf_name;
   if (m_Version >= 4) {
     stmf_name = pEncryptDict->GetStringFor("StmF");
     strf_name = pEncryptDict->GetStringFor("StrF");
@@ -203,14 +218,6 @@ bool CPDF_SecurityHandler::LoadDict(CPDF_Dictionary* pEncryptDict,
   return true;
 }
 
-bool CPDF_SecurityHandler::GetCryptInfo(int& cipher,
-                                        const uint8_t*& buffer,
-                                        int& keylen) {
-  cipher = m_Cipher;
-  buffer = m_EncryptKey;
-  keylen = m_KeyLen;
-  return true;
-}
 #define FX_GET_32WORD(n, b, i)                                        \
   {                                                                   \
     (n) = (uint32_t)(                                                 \
@@ -229,14 +236,13 @@ int BigOrder64BitsMod3(uint8_t* data) {
   return (int)ret;
 }
 
-void Revision6_Hash(const uint8_t* password,
-                    uint32_t size,
+void Revision6_Hash(const ByteString& password,
                     const uint8_t* salt,
                     const uint8_t* vector,
                     uint8_t* hash) {
-  CRYPT_sha256_context sha;
+  CRYPT_sha2_context sha;
   CRYPT_SHA256Start(&sha);
-  CRYPT_SHA256Update(&sha, password, size);
+  CRYPT_SHA256Update(&sha, password.raw_str(), password.GetLength());
   CRYPT_SHA256Update(&sha, salt, 8);
   if (vector)
     CRYPT_SHA256Update(&sha, vector, 48);
@@ -244,35 +250,37 @@ void Revision6_Hash(const uint8_t* password,
   uint8_t digest[32];
   CRYPT_SHA256Finish(&sha, digest);
 
-  CFX_ByteTextBuf buf;
+  std::vector<uint8_t> buf;
   uint8_t* input = digest;
   uint8_t* key = input;
   uint8_t* iv = input + 16;
-  uint8_t* E = buf.GetBuffer();
-  int iBufLen = buf.GetLength();
-  CFX_ByteTextBuf interDigest;
+  uint8_t* E = nullptr;
+  int iBufLen = 0;
+  std::vector<uint8_t> interDigest;
   int i = 0;
   int iBlockSize = 32;
-  uint8_t* aes = FX_Alloc(uint8_t, 2048);
+  CRYPT_aes_context aes;
+  memset(&aes, 0, sizeof(aes));
   while (i < 64 || i < E[iBufLen - 1] + 32) {
-    int iRoundSize = size + iBlockSize;
+    int iRoundSize = password.GetLength() + iBlockSize;
     if (vector) {
       iRoundSize += 48;
     }
     iBufLen = iRoundSize * 64;
-    buf.EstimateSize(iBufLen);
-    E = buf.GetBuffer();
-    CFX_ByteTextBuf content;
+    buf.resize(iBufLen);
+    E = buf.data();
+    std::vector<uint8_t> content;
     for (int j = 0; j < 64; ++j) {
-      content.AppendBlock(password, size);
-      content.AppendBlock(input, iBlockSize);
+      content.insert(std::end(content), password.raw_str(),
+                     password.raw_str() + password.GetLength());
+      content.insert(std::end(content), input, input + iBlockSize);
       if (vector) {
-        content.AppendBlock(vector, 48);
+        content.insert(std::end(content), vector, vector + 48);
       }
     }
-    CRYPT_AESSetKey(aes, 16, key, 16, true);
-    CRYPT_AESSetIV(aes, iv);
-    CRYPT_AESEncrypt(aes, E, content.GetBuffer(), iBufLen);
+    CRYPT_AESSetKey(&aes, 16, key, 16, true);
+    CRYPT_AESSetIV(&aes, iv);
+    CRYPT_AESEncrypt(&aes, E, content.data(), iBufLen);
     int iHash = 0;
     switch (BigOrder64BitsMod3(E)) {
       case 0:
@@ -288,8 +296,8 @@ void Revision6_Hash(const uint8_t* password,
         iBlockSize = 64;
         break;
     }
-    interDigest.EstimateSize(iBlockSize);
-    input = interDigest.GetBuffer();
+    interDigest.resize(iBlockSize);
+    input = interDigest.data();
     if (iHash == 0) {
       CRYPT_SHA256Generate(E, iBufLen, input);
     } else if (iHash == 1) {
@@ -301,40 +309,37 @@ void Revision6_Hash(const uint8_t* password,
     iv = input + 16;
     ++i;
   }
-  FX_Free(aes);
   if (hash) {
     memcpy(hash, input, 32);
   }
 }
 
-bool CPDF_SecurityHandler::AES256_CheckPassword(const uint8_t* password,
-                                                uint32_t size,
+bool CPDF_SecurityHandler::AES256_CheckPassword(const ByteString& password,
                                                 bool bOwner,
                                                 uint8_t* key) {
   if (!m_pEncryptDict)
     return false;
 
-  CFX_ByteString okey = m_pEncryptDict->GetStringFor("O");
+  ByteString okey = m_pEncryptDict->GetStringFor("O");
   if (okey.GetLength() < 48)
     return false;
 
-  CFX_ByteString ukey = m_pEncryptDict->GetStringFor("U");
+  ByteString ukey = m_pEncryptDict->GetStringFor("U");
   if (ukey.GetLength() < 48)
     return false;
 
   const uint8_t* pkey = bOwner ? okey.raw_str() : ukey.raw_str();
-  CRYPT_sha256_context sha;
+  CRYPT_sha2_context sha;
   uint8_t digest[32];
   if (m_Revision >= 6) {
-    Revision6_Hash(password, size, (const uint8_t*)pkey + 32,
+    Revision6_Hash(password, (const uint8_t*)pkey + 32,
                    bOwner ? ukey.raw_str() : nullptr, digest);
   } else {
     CRYPT_SHA256Start(&sha);
-    CRYPT_SHA256Update(&sha, password, size);
+    CRYPT_SHA256Update(&sha, password.raw_str(), password.GetLength());
     CRYPT_SHA256Update(&sha, pkey + 32, 8);
     if (bOwner)
       CRYPT_SHA256Update(&sha, ukey.raw_str(), 48);
-
     CRYPT_SHA256Finish(&sha, digest);
   }
   if (memcmp(digest, pkey, 32) != 0)
@@ -344,32 +349,30 @@ bool CPDF_SecurityHandler::AES256_CheckPassword(const uint8_t* password,
     return true;
 
   if (m_Revision >= 6) {
-    Revision6_Hash(password, size, (const uint8_t*)pkey + 40,
+    Revision6_Hash(password, (const uint8_t*)pkey + 40,
                    bOwner ? ukey.raw_str() : nullptr, digest);
   } else {
     CRYPT_SHA256Start(&sha);
-    CRYPT_SHA256Update(&sha, password, size);
+    CRYPT_SHA256Update(&sha, password.raw_str(), password.GetLength());
     CRYPT_SHA256Update(&sha, pkey + 40, 8);
     if (bOwner)
       CRYPT_SHA256Update(&sha, ukey.raw_str(), 48);
-
     CRYPT_SHA256Finish(&sha, digest);
   }
-  CFX_ByteString ekey = m_pEncryptDict
-                            ? m_pEncryptDict->GetStringFor(bOwner ? "OE" : "UE")
-                            : CFX_ByteString();
+  ByteString ekey = m_pEncryptDict->GetStringFor(bOwner ? "OE" : "UE");
   if (ekey.GetLength() < 32)
     return false;
 
-  std::vector<uint8_t> aes(2048);
-  CRYPT_AESSetKey(aes.data(), 16, digest, 32, false);
+  CRYPT_aes_context aes;
+  memset(&aes, 0, sizeof(aes));
+  CRYPT_AESSetKey(&aes, 16, digest, 32, false);
   uint8_t iv[16];
   memset(iv, 0, 16);
-  CRYPT_AESSetIV(aes.data(), iv);
-  CRYPT_AESDecrypt(aes.data(), key, ekey.raw_str(), 32);
-  CRYPT_AESSetKey(aes.data(), 16, key, 32, false);
-  CRYPT_AESSetIV(aes.data(), iv);
-  CFX_ByteString perms = m_pEncryptDict->GetStringFor("Perms");
+  CRYPT_AESSetIV(&aes, iv);
+  CRYPT_AESDecrypt(&aes, key, ekey.raw_str(), 32);
+  CRYPT_AESSetKey(&aes, 16, key, 32, false);
+  CRYPT_AESSetIV(&aes, iv);
+  ByteString perms = m_pEncryptDict->GetStringFor("Perms");
   if (perms.IsEmpty())
     return false;
 
@@ -379,7 +382,7 @@ bool CPDF_SecurityHandler::AES256_CheckPassword(const uint8_t* password,
       std::min(sizeof(perms_buf), static_cast<size_t>(perms.GetLength()));
   memcpy(perms_buf, perms.raw_str(), copy_len);
   uint8_t buf[16];
-  CRYPT_AESDecrypt(aes.data(), buf, perms_buf, 16);
+  CRYPT_AESDecrypt(&aes, buf, perms_buf, 16);
   if (buf[9] != 'a' || buf[10] != 'd' || buf[11] != 'b')
     return false;
 
@@ -393,36 +396,36 @@ bool CPDF_SecurityHandler::AES256_CheckPassword(const uint8_t* password,
   return buf[8] == 'F' || IsMetadataEncrypted();
 }
 
-bool CPDF_SecurityHandler::CheckPassword(const uint8_t* password,
-                                         uint32_t size,
+bool CPDF_SecurityHandler::CheckPassword(const ByteString& password,
                                          bool bOwner,
                                          uint8_t* key,
                                          int32_t key_len) {
   if (m_Revision >= 5)
-    return AES256_CheckPassword(password, size, bOwner, key);
+    return AES256_CheckPassword(password, bOwner, key);
 
   uint8_t keybuf[32];
   if (!key)
     key = keybuf;
 
   if (bOwner)
-    return CheckOwnerPassword(password, size, key, key_len);
+    return CheckOwnerPassword(password, key, key_len);
 
-  return CheckUserPassword(password, size, false, key, key_len) ||
-         CheckUserPassword(password, size, true, key, key_len);
+  return CheckUserPassword(password, false, key, key_len) ||
+         CheckUserPassword(password, true, key, key_len);
 }
-bool CPDF_SecurityHandler::CheckUserPassword(const uint8_t* password,
-                                             uint32_t pass_size,
+
+bool CPDF_SecurityHandler::CheckUserPassword(const ByteString& password,
                                              bool bIgnoreEncryptMeta,
                                              uint8_t* key,
                                              int32_t key_len) {
-  CalcEncryptKey(m_pEncryptDict, password, pass_size, key, key_len,
-                 bIgnoreEncryptMeta, m_pParser->GetIDArray());
-  CFX_ByteString ukey =
-      m_pEncryptDict ? m_pEncryptDict->GetStringFor("U") : CFX_ByteString();
+  CalcEncryptKey(m_pEncryptDict.Get(), password, key, key_len,
+                 bIgnoreEncryptMeta, m_FileId);
+  ByteString ukey =
+      m_pEncryptDict ? m_pEncryptDict->GetStringFor("U") : ByteString();
   if (ukey.GetLength() < 16) {
     return false;
   }
+
   uint8_t ukeybuf[32];
   if (m_Revision == 2) {
     memcpy(ukeybuf, defpasscode, 32);
@@ -444,10 +447,8 @@ bool CPDF_SecurityHandler::CheckUserPassword(const uint8_t* password,
     CRYPT_md5_context md5;
     CRYPT_MD5Start(&md5);
     CRYPT_MD5Update(&md5, defpasscode, 32);
-    CPDF_Array* pIdArray = m_pParser->GetIDArray();
-    if (pIdArray) {
-      CFX_ByteString id = pIdArray->GetStringAt(0);
-      CRYPT_MD5Update(&md5, (uint8_t*)id.c_str(), id.GetLength());
+    if (!m_FileId.IsEmpty()) {
+      CRYPT_MD5Update(&md5, (uint8_t*)m_FileId.c_str(), m_FileId.GetLength());
     }
     CRYPT_MD5Finish(&md5, ukeybuf);
     return memcmp(test, ukeybuf, 16) == 0;
@@ -455,13 +456,15 @@ bool CPDF_SecurityHandler::CheckUserPassword(const uint8_t* password,
   return memcmp(ukey.c_str(), ukeybuf, 16) == 0;
 }
 
-CFX_ByteString CPDF_SecurityHandler::GetUserPassword(const uint8_t* owner_pass,
-                                                     uint32_t pass_size,
-                                                     int32_t key_len) {
-  CFX_ByteString okey = m_pEncryptDict->GetStringFor("O");
+ByteString CPDF_SecurityHandler::GetUserPassword(
+    const ByteString& owner_password,
+    int32_t key_len) const {
+  ByteString okey = m_pEncryptDict->GetStringFor("O");
   uint8_t passcode[32];
   for (uint32_t i = 0; i < 32; i++) {
-    passcode[i] = i < pass_size ? owner_pass[i] : defpasscode[i - pass_size];
+    passcode[i] = i < owner_password.GetLength()
+                      ? owner_password[i]
+                      : defpasscode[i - owner_password.GetLength()];
   }
   uint8_t digest[16];
   CRYPT_MD5Generate(passcode, 32, digest);
@@ -499,53 +502,49 @@ CFX_ByteString CPDF_SecurityHandler::GetUserPassword(const uint8_t* owner_pass,
   while (len && defpasscode[len - 1] == okeybuf[len - 1]) {
     len--;
   }
-  return CFX_ByteString(okeybuf, len);
+  return ByteString(okeybuf, len);
 }
-bool CPDF_SecurityHandler::CheckOwnerPassword(const uint8_t* password,
-                                              uint32_t pass_size,
+
+bool CPDF_SecurityHandler::CheckOwnerPassword(const ByteString& password,
                                               uint8_t* key,
                                               int32_t key_len) {
-  CFX_ByteString user_pass = GetUserPassword(password, pass_size, key_len);
-  if (CheckUserPassword(user_pass.raw_str(), user_pass.GetLength(), false, key,
-                        key_len)) {
+  ByteString user_pass = GetUserPassword(password, key_len);
+  if (CheckUserPassword(user_pass, false, key, key_len))
     return true;
-  }
-  return CheckUserPassword(user_pass.raw_str(), user_pass.GetLength(), true,
-                           key, key_len);
+  return CheckUserPassword(user_pass, true, key, key_len);
 }
 
 bool CPDF_SecurityHandler::IsMetadataEncrypted() const {
   return m_pEncryptDict->GetBooleanFor("EncryptMetadata", true);
 }
 
-void CPDF_SecurityHandler::OnCreate(CPDF_Dictionary* pEncryptDict,
-                                    CPDF_Array* pIdArray,
-                                    const uint8_t* user_pass,
-                                    uint32_t user_size,
-                                    const uint8_t* owner_pass,
-                                    uint32_t owner_size,
-                                    bool bDefault,
-                                    uint32_t type) {
-  int cipher = 0, key_len = 0;
-  if (!LoadDict(pEncryptDict, type, cipher, key_len)) {
+void CPDF_SecurityHandler::OnCreateInternal(CPDF_Dictionary* pEncryptDict,
+                                            const CPDF_Array* pIdArray,
+                                            const ByteString& user_password,
+                                            const ByteString& owner_password,
+                                            bool bDefault) {
+  ASSERT(pEncryptDict);
+
+  int cipher = 0;
+  int key_len = 0;
+  if (!LoadDict(pEncryptDict, cipher, key_len)) {
     return;
   }
-  if (bDefault && (!owner_pass || owner_size == 0)) {
-    owner_pass = user_pass;
-    owner_size = user_size;
-  }
+  ByteString owner_password_copy = owner_password;
+  if (bDefault && owner_password.IsEmpty())
+    owner_password_copy = user_password;
+
   if (m_Revision >= 5) {
-    int t = (int)time(nullptr);
-    CRYPT_sha256_context sha;
+    int t = static_cast<int>(FXSYS_time(nullptr));
+    CRYPT_sha2_context sha;
     CRYPT_SHA256Start(&sha);
     CRYPT_SHA256Update(&sha, (uint8_t*)&t, sizeof t);
     CRYPT_SHA256Update(&sha, m_EncryptKey, 32);
     CRYPT_SHA256Update(&sha, (uint8_t*)"there", 5);
     CRYPT_SHA256Finish(&sha, m_EncryptKey);
-    AES256_SetPassword(pEncryptDict, user_pass, user_size, false, m_EncryptKey);
+    AES256_SetPassword(pEncryptDict, user_password, false, m_EncryptKey);
     if (bDefault) {
-      AES256_SetPassword(pEncryptDict, owner_pass, owner_size, true,
-                         m_EncryptKey);
+      AES256_SetPassword(pEncryptDict, owner_password_copy, true, m_EncryptKey);
       AES256_SetPerms(pEncryptDict, m_Permissions,
                       pEncryptDict->GetBooleanFor("EncryptMetadata", true),
                       m_EncryptKey);
@@ -555,8 +554,9 @@ void CPDF_SecurityHandler::OnCreate(CPDF_Dictionary* pEncryptDict,
   if (bDefault) {
     uint8_t passcode[32];
     for (uint32_t i = 0; i < 32; i++) {
-      passcode[i] =
-          i < owner_size ? owner_pass[i] : defpasscode[i - owner_size];
+      passcode[i] = i < owner_password_copy.GetLength()
+                        ? owner_password_copy[i]
+                        : defpasscode[i - owner_password_copy.GetLength()];
     }
     uint8_t digest[16];
     CRYPT_MD5Generate(passcode, 32, digest);
@@ -567,7 +567,9 @@ void CPDF_SecurityHandler::OnCreate(CPDF_Dictionary* pEncryptDict,
     uint8_t enckey[32];
     memcpy(enckey, digest, key_len);
     for (uint32_t i = 0; i < 32; i++) {
-      passcode[i] = i < user_size ? user_pass[i] : defpasscode[i - user_size];
+      passcode[i] = i < user_password.GetLength()
+                        ? user_password[i]
+                        : defpasscode[i - user_password.GetLength()];
     }
     CRYPT_ArcFourCryptBlock(passcode, 32, enckey, key_len);
     uint8_t tempkey[32];
@@ -578,24 +580,27 @@ void CPDF_SecurityHandler::OnCreate(CPDF_Dictionary* pEncryptDict,
         CRYPT_ArcFourCryptBlock(passcode, 32, tempkey, key_len);
       }
     }
-    pEncryptDict->SetNewFor<CPDF_String>("O", CFX_ByteString(passcode, 32),
-                                         false);
+    pEncryptDict->SetNewFor<CPDF_String>("O", ByteString(passcode, 32), false);
   }
-  CalcEncryptKey(m_pEncryptDict, (uint8_t*)user_pass, user_size, m_EncryptKey,
-                 key_len, false, pIdArray);
+
+  ByteString fileId;
+  if (pIdArray) {
+    fileId = pIdArray->GetStringAt(0);
+  }
+
+  CalcEncryptKey(m_pEncryptDict.Get(), user_password, m_EncryptKey, key_len,
+                 false, fileId);
   if (m_Revision < 3) {
     uint8_t tempbuf[32];
     memcpy(tempbuf, defpasscode, 32);
     CRYPT_ArcFourCryptBlock(tempbuf, 32, m_EncryptKey, key_len);
-    pEncryptDict->SetNewFor<CPDF_String>("U", CFX_ByteString(tempbuf, 32),
-                                         false);
+    pEncryptDict->SetNewFor<CPDF_String>("U", ByteString(tempbuf, 32), false);
   } else {
     CRYPT_md5_context md5;
     CRYPT_MD5Start(&md5);
     CRYPT_MD5Update(&md5, defpasscode, 32);
-    if (pIdArray) {
-      CFX_ByteString id = pIdArray->GetStringAt(0);
-      CRYPT_MD5Update(&md5, (uint8_t*)id.c_str(), id.GetLength());
+    if (!fileId.IsEmpty()) {
+      CRYPT_MD5Update(&md5, (uint8_t*)fileId.c_str(), fileId.GetLength());
     }
     uint8_t digest[32];
     CRYPT_MD5Finish(&md5, digest);
@@ -608,31 +613,27 @@ void CPDF_SecurityHandler::OnCreate(CPDF_Dictionary* pEncryptDict,
       CRYPT_ArcFourCryptBlock(digest, 16, tempkey, key_len);
     }
     CRYPT_MD5Generate(digest, 16, digest + 16);
-    pEncryptDict->SetNewFor<CPDF_String>("U", CFX_ByteString(digest, 32),
-                                         false);
+    pEncryptDict->SetNewFor<CPDF_String>("U", ByteString(digest, 32), false);
   }
 }
+
 void CPDF_SecurityHandler::OnCreate(CPDF_Dictionary* pEncryptDict,
-                                    CPDF_Array* pIdArray,
-                                    const uint8_t* user_pass,
-                                    uint32_t user_size,
-                                    const uint8_t* owner_pass,
-                                    uint32_t owner_size,
-                                    uint32_t type) {
-  OnCreate(pEncryptDict, pIdArray, user_pass, user_size, owner_pass, owner_size,
-           true, type);
+                                    const CPDF_Array* pIdArray,
+                                    const ByteString& user_password,
+                                    const ByteString& owner_password) {
+  OnCreateInternal(pEncryptDict, pIdArray, user_password, owner_password, true);
+  InitCryptoHandler();
 }
+
 void CPDF_SecurityHandler::OnCreate(CPDF_Dictionary* pEncryptDict,
-                                    CPDF_Array* pIdArray,
-                                    const uint8_t* user_pass,
-                                    uint32_t user_size,
-                                    uint32_t type) {
-  OnCreate(pEncryptDict, pIdArray, user_pass, user_size, nullptr, 0, false,
-           type);
+                                    const CPDF_Array* pIdArray,
+                                    const ByteString& user_password) {
+  OnCreateInternal(pEncryptDict, pIdArray, user_password, ByteString(), false);
+  InitCryptoHandler();
 }
+
 void CPDF_SecurityHandler::AES256_SetPassword(CPDF_Dictionary* pEncryptDict,
-                                              const uint8_t* password,
-                                              uint32_t size,
+                                              const ByteString& password,
                                               bool bOwner,
                                               const uint8_t* key) {
   CRYPT_sha1_context sha;
@@ -643,15 +644,15 @@ void CPDF_SecurityHandler::AES256_SetPassword(CPDF_Dictionary* pEncryptDict,
   uint8_t digest[20];
   CRYPT_SHA1Finish(&sha, digest);
 
-  CFX_ByteString ukey = pEncryptDict->GetStringFor("U");
-  CRYPT_sha256_context sha2;
+  ByteString ukey = pEncryptDict->GetStringFor("U");
+  CRYPT_sha2_context sha2;
   uint8_t digest1[48];
   if (m_Revision >= 6) {
-    Revision6_Hash(password, size, digest, bOwner ? ukey.raw_str() : nullptr,
+    Revision6_Hash(password, digest, bOwner ? ukey.raw_str() : nullptr,
                    digest1);
   } else {
     CRYPT_SHA256Start(&sha2);
-    CRYPT_SHA256Update(&sha2, password, size);
+    CRYPT_SHA256Update(&sha2, password.raw_str(), password.GetLength());
     CRYPT_SHA256Update(&sha2, digest, 8);
     if (bOwner) {
       CRYPT_SHA256Update(&sha2, ukey.raw_str(), ukey.GetLength());
@@ -660,28 +661,28 @@ void CPDF_SecurityHandler::AES256_SetPassword(CPDF_Dictionary* pEncryptDict,
   }
   memcpy(digest1 + 32, digest, 16);
   pEncryptDict->SetNewFor<CPDF_String>(bOwner ? "O" : "U",
-                                       CFX_ByteString(digest1, 48), false);
+                                       ByteString(digest1, 48), false);
   if (m_Revision >= 6) {
-    Revision6_Hash(password, size, digest + 8,
-                   bOwner ? ukey.raw_str() : nullptr, digest1);
+    Revision6_Hash(password, digest + 8, bOwner ? ukey.raw_str() : nullptr,
+                   digest1);
   } else {
     CRYPT_SHA256Start(&sha2);
-    CRYPT_SHA256Update(&sha2, password, size);
+    CRYPT_SHA256Update(&sha2, password.raw_str(), password.GetLength());
     CRYPT_SHA256Update(&sha2, digest + 8, 8);
     if (bOwner) {
       CRYPT_SHA256Update(&sha2, ukey.raw_str(), ukey.GetLength());
     }
     CRYPT_SHA256Finish(&sha2, digest1);
   }
-  uint8_t* aes = FX_Alloc(uint8_t, 2048);
-  CRYPT_AESSetKey(aes, 16, digest1, 32, true);
+  CRYPT_aes_context aes;
+  memset(&aes, 0, sizeof(aes));
+  CRYPT_AESSetKey(&aes, 16, digest1, 32, true);
   uint8_t iv[16];
   memset(iv, 0, 16);
-  CRYPT_AESSetIV(aes, iv);
-  CRYPT_AESEncrypt(aes, digest1, key, 32);
-  FX_Free(aes);
+  CRYPT_AESSetIV(&aes, iv);
+  CRYPT_AESEncrypt(&aes, digest1, key, 32);
   pEncryptDict->SetNewFor<CPDF_String>(bOwner ? "OE" : "UE",
-                                       CFX_ByteString(digest1, 32), false);
+                                       ByteString(digest1, 32), false);
 }
 
 void CPDF_SecurityHandler::AES256_SetPerms(CPDF_Dictionary* pEncryptDict,
@@ -701,13 +702,21 @@ void CPDF_SecurityHandler::AES256_SetPerms(CPDF_Dictionary* pEncryptDict,
   buf[9] = 'a';
   buf[10] = 'd';
   buf[11] = 'b';
-  uint8_t* aes = FX_Alloc(uint8_t, 2048);
-  CRYPT_AESSetKey(aes, 16, key, 32, true);
-  uint8_t iv[16], buf1[16];
+
+  CRYPT_aes_context aes;
+  memset(&aes, 0, sizeof(aes));
+  CRYPT_AESSetKey(&aes, 16, key, 32, true);
+
+  uint8_t iv[16];
   memset(iv, 0, 16);
-  CRYPT_AESSetIV(aes, iv);
-  CRYPT_AESEncrypt(aes, buf1, buf, 16);
-  FX_Free(aes);
-  pEncryptDict->SetNewFor<CPDF_String>("Perms", CFX_ByteString(buf1, 16),
-                                       false);
+  CRYPT_AESSetIV(&aes, iv);
+
+  uint8_t buf1[16];
+  CRYPT_AESEncrypt(&aes, buf1, buf, 16);
+  pEncryptDict->SetNewFor<CPDF_String>("Perms", ByteString(buf1, 16), false);
+}
+
+void CPDF_SecurityHandler::InitCryptoHandler() {
+  m_pCryptoHandler =
+      pdfium::MakeUnique<CPDF_CryptoHandler>(m_Cipher, m_EncryptKey, m_KeyLen);
 }
