@@ -26,13 +26,22 @@ enum FXCODEC_STATUS {
 #ifdef PDF_ENABLE_XFA
 enum FXCODEC_IMAGE_TYPE {
   FXCODEC_IMAGE_UNKNOWN = 0,
-  FXCODEC_IMAGE_BMP,
   FXCODEC_IMAGE_JPG,
+#ifdef PDF_ENABLE_XFA_BMP
+  FXCODEC_IMAGE_BMP,
+#endif  // PDF_ENABLE_XFA_BMP
+#ifdef PDF_ENABLE_XFA_PNG
   FXCODEC_IMAGE_PNG,
+#endif  // PDF_ENABLE_XFA_PNG
+#ifdef PDF_ENABLE_XFA_GIF
   FXCODEC_IMAGE_GIF,
-  FXCODEC_IMAGE_TIF,
+#endif  // PDF_ENABLE_XFA_GIF
+#ifdef PDF_ENABLE_XFA_TIFF
+  FXCODEC_IMAGE_TIFF,
+#endif  // PDF_ENABLE_XFA_TIFF
   FXCODEC_IMAGE_MAX
 };
+
 enum FXCODEC_RESUNIT {
   FXCODEC_RESUNIT_NONE = 0,
   FXCODEC_RESUNIT_INCH,
