@@ -7,15 +7,19 @@
 #ifndef CORE_FXCRT_CFX_FILEACCESS_WINDOWS_H_
 #define CORE_FXCRT_CFX_FILEACCESS_WINDOWS_H_
 
-#include "core/fxcrt/ifx_fileaccess.h"
+#include "core/fxcrt/fileaccess_iface.h"
+#include "core/fxcrt/fx_system.h"
 
-#if _FX_PLATFORM_ == _FX_PLATFORM_WINDOWS_
-class CFX_FileAccess_Windows : public IFX_FileAccess {
+#if _FX_PLATFORM_ != _FX_PLATFORM_WINDOWS_
+#error "Included on the wrong platform"
+#endif
+
+class CFX_FileAccess_Windows final : public FileAccessIface {
  public:
   CFX_FileAccess_Windows();
   ~CFX_FileAccess_Windows() override;
 
-  // IFX_FileAccess
+  // FileAccessIface
   bool Open(const ByteStringView& fileName, uint32_t dwMode) override;
   bool Open(const WideStringView& fileName, uint32_t dwMode) override;
   void Close() override;
@@ -34,6 +38,5 @@ class CFX_FileAccess_Windows : public IFX_FileAccess {
  protected:
   void* m_hFile;
 };
-#endif
 
 #endif  // CORE_FXCRT_CFX_FILEACCESS_WINDOWS_H_
