@@ -16,7 +16,7 @@ class CPDFSDK_InterForm;
 class CPDFSDK_PageView;
 class CXFA_FFWidget;
 
-class CPDFSDK_XFAWidget : public CPDFSDK_Annot {
+class CPDFSDK_XFAWidget final : public CPDFSDK_Annot {
  public:
   CPDFSDK_XFAWidget(CXFA_FFWidget* pAnnot,
                     CPDFSDK_PageView* pPageView,
@@ -36,4 +36,9 @@ class CPDFSDK_XFAWidget : public CPDFSDK_Annot {
   UnownedPtr<CXFA_FFWidget> m_hXFAWidget;
 };
 
+inline CPDFSDK_XFAWidget* ToXFAWidget(CPDFSDK_Annot* pAnnot) {
+  return pAnnot && pAnnot->GetAnnotSubtype() == CPDF_Annot::Subtype::XFAWIDGET
+      ? static_cast<CPDFSDK_XFAWidget*>(pAnnot) : nullptr;
+}
+
 #endif  // FPDFSDK_CPDFSDK_XFAWIDGET_H_
