@@ -12,24 +12,23 @@
 #include "core/fxge/fx_dib.h"
 #include "fxbarcode/cbc_codebase.h"
 
-class CBC_PDF417I : public CBC_CodeBase {
+class CBC_PDF417Writer;
+
+class CBC_PDF417I final : public CBC_CodeBase {
  public:
   CBC_PDF417I();
   ~CBC_PDF417I() override;
 
-  // CBC_CodeBase::
-  bool Encode(const CFX_WideStringC& contents,
-              bool isDevice,
-              int32_t& e) override;
+  // CBC_CodeBase:
+  bool Encode(const WideStringView& contents) override;
   bool RenderDevice(CFX_RenderDevice* device,
-                    const CFX_Matrix* matrix,
-                    int32_t& e) override;
-  bool RenderBitmap(CFX_RetainPtr<CFX_DIBitmap>& pOutBitmap,
-                    int32_t& e) override;
+                    const CFX_Matrix* matrix) override;
   BC_TYPE GetType() override;
 
   bool SetErrorCorrectionLevel(int32_t level);
-  void SetTruncated(bool truncated);
+
+ private:
+  CBC_PDF417Writer* GetPDF417Writer();
 };
 
 #endif  // FXBARCODE_CBC_PDF417I_H_
