@@ -14,26 +14,21 @@
 
 class CBC_QRCodeWriter;
 
-class CBC_QRCode : public CBC_CodeBase {
+class CBC_QRCode final : public CBC_CodeBase {
  public:
   CBC_QRCode();
   ~CBC_QRCode() override;
 
   // CBC_CodeBase:
-  bool Encode(const CFX_WideStringC& contents,
-              bool isDevice,
-              int32_t& e) override;
+  bool Encode(const WideStringView& contents) override;
   bool RenderDevice(CFX_RenderDevice* device,
-                    const CFX_Matrix* matrix,
-                    int32_t& e) override;
-  bool RenderBitmap(CFX_RetainPtr<CFX_DIBitmap>& pOutBitmap,
-                    int32_t& e) override;
+                    const CFX_Matrix* matrix) override;
   BC_TYPE GetType() override;
 
   bool SetErrorCorrectionLevel(int32_t level);
 
  private:
-  CBC_QRCodeWriter* writer();
+  CBC_QRCodeWriter* GetQRCodeWriter();
 };
 
 #endif  // FXBARCODE_CBC_QRCODE_H_
