@@ -7,12 +7,12 @@
 #ifndef FXJS_XFA_CJX_TEMPLATE_H_
 #define FXJS_XFA_CJX_TEMPLATE_H_
 
-#include "fxjs/CJX_Define.h"
+#include "fxjs/jse_define.h"
 #include "fxjs/xfa/cjx_model.h"
 
 class CXFA_Template;
 
-class CJX_Template : public CJX_Model {
+class CJX_Template final : public CJX_Model {
  public:
   explicit CJX_Template(CXFA_Template* tmpl);
   ~CJX_Template() override;
@@ -20,12 +20,12 @@ class CJX_Template : public CJX_Model {
   /* The docs list a |createNode| method on Template but that method already
    * exists on Model, also the |createNode| docs say it exists on Model not
    * on Template so I'm leaving |createNode| out of the template methods. */
-  JS_METHOD(execCalculate, CJX_Template);
-  JS_METHOD(execInitialize, CJX_Template);
-  JS_METHOD(execValidate, CJX_Template);
-  JS_METHOD(formNodes, CJX_Template);
-  JS_METHOD(recalculate, CJX_Template);
-  JS_METHOD(remerge, CJX_Template);
+  JSE_METHOD(execCalculate, CJX_Template);
+  JSE_METHOD(execInitialize, CJX_Template);
+  JSE_METHOD(execValidate, CJX_Template);
+  JSE_METHOD(formNodes, CJX_Template);
+  JSE_METHOD(recalculate, CJX_Template);
+  JSE_METHOD(remerge, CJX_Template);
 
  private:
   static const CJX_MethodSpec MethodSpecs[];
