@@ -7,17 +7,18 @@
 #ifndef FXJS_XFA_CJX_USER_H_
 #define FXJS_XFA_CJX_USER_H_
 
+#include "fxjs/jse_define.h"
 #include "fxjs/xfa/cjx_textnode.h"
 
 class CXFA_User;
 
-class CJX_User : public CJX_TextNode {
+class CJX_User final : public CJX_TextNode {
  public:
   explicit CJX_User(CXFA_User* node);
   ~CJX_User() override;
 
-  JS_PROP(use);
-  JS_PROP(usehref);
+  JSE_PROP(use);
+  JSE_PROP(usehref);
 };
 
 #endif  // FXJS_XFA_CJX_USER_H_
