@@ -7,11 +7,11 @@
 #ifndef XFA_FXFA_CXFA_FFIMAGE_H_
 #define XFA_FXFA_CXFA_FFIMAGE_H_
 
-#include "xfa/fxfa/cxfa_ffdraw.h"
+#include "xfa/fxfa/cxfa_ffwidget.h"
 
-class CXFA_FFImage : public CXFA_FFDraw {
+class CXFA_FFImage final : public CXFA_FFWidget {
  public:
-  explicit CXFA_FFImage(CXFA_WidgetAcc* pDataAcc);
+  explicit CXFA_FFImage(CXFA_Node* pNode);
   ~CXFA_FFImage() override;
 
   // CXFA_FFWidget
@@ -20,7 +20,6 @@ class CXFA_FFImage : public CXFA_FFDraw {
                     uint32_t dwStatus) override;
   bool IsLoaded() override;
   bool LoadWidget() override;
-  void UnloadWidget() override;
 };
 
 #endif  // XFA_FXFA_CXFA_FFIMAGE_H_
