@@ -8,13 +8,12 @@
 #define XFA_FXFA_CXFA_TEXTPROVIDER_H_
 
 #include "core/fxcrt/fx_string.h"
+#include "third_party/base/optional.h"
 #include "xfa/fxfa/cxfa_textlayout.h"
-#include "xfa/fxfa/cxfa_widgetacc.h"
-#include "xfa/fxfa/parser/cxfa_font.h"
-#include "xfa/fxfa/parser/cxfa_para.h"
 
-class CXFA_FFDoc;
+class CXFA_Font;
 class CXFA_Node;
+class CXFA_Para;
 
 enum XFA_TEXTPROVIDERTYPE {
   XFA_TEXTPROVIDERTYPE_Text,
@@ -26,24 +25,20 @@ enum XFA_TEXTPROVIDERTYPE {
 
 class CXFA_TextProvider {
  public:
-  CXFA_TextProvider(CXFA_WidgetAcc* pWidgetAcc, XFA_TEXTPROVIDERTYPE eType)
-      : m_pWidgetAcc(pWidgetAcc), m_eType(eType) {
-    ASSERT(m_pWidgetAcc);
+  CXFA_TextProvider(CXFA_Node* pNode, XFA_TEXTPROVIDERTYPE eType)
+      : m_pNode(pNode), m_eType(eType) {
+    ASSERT(m_pNode);
   }
   ~CXFA_TextProvider() {}
 
   CXFA_Node* GetTextNode(bool& bRichText);
-  CXFA_Para GetParaNode();
-  CXFA_Font GetFontNode();
-  bool IsCheckButtonAndAutoWidth();
-  CXFA_FFDoc* GetDocNode() { return m_pWidgetAcc->GetDoc(); }
-  bool GetEmbbedObj(bool bURI,
-                    bool bRaw,
-                    const WideString& wsAttr,
-                    WideString& wsValue);
+  CXFA_Para* GetParaIfExists();
+  CXFA_Font* GetFontIfExists();
+  bool IsCheckButtonAndAutoWidth() const;
+  Optional<WideString> GetEmbeddedObj(const WideString& wsAttr) const;
 
  private:
-  CXFA_WidgetAcc* m_pWidgetAcc;
+  CXFA_Node* m_pNode;  // Raw, this class owned by tree node.
   XFA_TEXTPROVIDERTYPE m_eType;
 };
 
